package sun.instrument;

import java.security.ProtectionDomain;
import java.lang.instrument.ClassFileTransformer;

public class TransformerManager
{
    private TransformerManager.TransformerInfo[] mTransformerList;
    private boolean mIsRetransformable;
    
    TransformerManager(boolean mIsRetransformable) {
        this.mTransformerList = new TransformerManager.TransformerInfo[0];
        this.mIsRetransformable = mIsRetransformable;
    }
    
    boolean isRetransformable() {
        return this.mIsRetransformable;
    }
    
    public synchronized void addTransformer(ClassFileTransformer classFileTransformer) {
        TransformerManager.TransformerInfo[] mTransformerList = this.mTransformerList;
        TransformerManager.TransformerInfo[] mTransformerList2 = new TransformerManager.TransformerInfo[mTransformerList.length + 1];
        System.arraycopy(mTransformerList, 0, mTransformerList2, 0, mTransformerList.length);
        mTransformerList2[mTransformerList.length] = new TransformerManager.TransformerInfo(classFileTransformer);
        this.mTransformerList = mTransformerList2;
    }
    
    public synchronized boolean removeTransformer(ClassFileTransformer classFileTransformer) {
        boolean n = false;
        TransformerManager.TransformerInfo[] mTransformerList = this.mTransformerList;
        int length = mTransformerList.length;
        int n2 = length - 1;
        int n3 = 0;
        int i = length - 1;
        while (i >= 0) {
            if (mTransformerList[i].transformer() == classFileTransformer) {
                n = true;
                n3 = i;
                break;
            }
            --i;
        }
        if (!(n == 0)) {
            TransformerManager.TransformerInfo[] mTransformerList2 = new TransformerManager.TransformerInfo[n2];
            if (n3 > 0) {
                System.arraycopy(mTransformerList, 0, mTransformerList2, 0, n3);
            }
            if (n3 < n2) {
                System.arraycopy(mTransformerList, n3 + 1, mTransformerList2, n3, n2 - n3);
            }
            this.mTransformerList = mTransformerList2;
        }
        return n;
    }
    
    synchronized boolean includesTransformer(ClassFileTransformer classFileTransformer) {
        TransformerManager.TransformerInfo[] mTransformerList = this.mTransformerList;
        int length = mTransformerList.length;
        int i = 0;
        while (i < length) {
            if (mTransformerList[i].transformer() == classFileTransformer) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private TransformerManager.TransformerInfo[] getSnapshotTransformerList() {
        return this.mTransformerList;
    }
    
    public byte[] transform(ClassLoader loader, String className, Class<?> classBeingRedefined, ProtectionDomain protectionDomain, byte[] classfileBuffer) {
        boolean n = false;
        TransformerManager.TransformerInfo[] snapshotTransformerList = this.getSnapshotTransformerList();
        for (int i = 0; i < snapshotTransformerList.length; ++i) {
            ClassFileTransformer transformer = snapshotTransformerList[i].transformer();
            byte[] transform = null;
            try {
                transform = transformer.transform(loader, className, classBeingRedefined, protectionDomain, classfileBuffer);
            }
            catch (Throwable t) {}
            if (transform != null) {
                n = true;
                classfileBuffer = transform;
            }
        }
        if (!(n == 0)) {
            return classfileBuffer;
        }
        classfileBuffer = null;
        return classfileBuffer;
    }
    
    int getTransformerCount() {
        return this.getSnapshotTransformerList().length;
    }
    
    boolean setNativeMethodPrefix(ClassFileTransformer classFileTransformer, String prefix) {
        TransformerManager.TransformerInfo[] snapshotTransformerList = this.getSnapshotTransformerList();
        int i = 0;
        while (i < snapshotTransformerList.length) {
            TransformerManager.TransformerInfo transformerInfo = snapshotTransformerList[i];
            if (transformerInfo.transformer() == classFileTransformer) {
                transformerInfo.setPrefix(prefix);
                return true;
            }
            ++i;
        }
        return false;
    }
    
    String[] getNativeMethodPrefixes() {
        TransformerManager.TransformerInfo[] snapshotTransformerList = this.getSnapshotTransformerList();
        String[] array = new String[snapshotTransformerList.length];
        for (int i = 0; i < snapshotTransformerList.length; ++i) {
            array[i] = snapshotTransformerList[i].getPrefix();
        }
        return array;
    }
    
    private class TransformerInfo
    {
        final ClassFileTransformer mTransformer;
        String mPrefix;
        
        TransformerInfo(ClassFileTransformer mTransformer) {
            this.mTransformer = mTransformer;
            this.mPrefix = null;
        }
        
        ClassFileTransformer transformer() {
            return this.mTransformer;
        }
        
        String getPrefix() {
            return this.mPrefix;
        }
        
        void setPrefix(String mPrefix) {
            this.mPrefix = mPrefix;
        }
    }
}
